from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class FormationConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'formation'

    def ready(self):
        migration_override = getattr(settings, 'MIGRATION_MODULES', {}).get('formation')
        if not migration_override:
            raise ImproperlyConfigured('formation needs to be added to a MIGRATION_MODULES setting')



from django.core.mail import send_mail
from formation.blocks import (
    BaseFormBlock, BooleanFieldBlock, SubmitButtonBlock, TextFieldBlock
)
from wagtail.blocks import EmailBlock, RichTextBlock, StreamBlock
from wagtail.images.blocks import ImageChooserBlock


class MessageFormBlock(BaseFormBlock):
    send_emails_to = EmailBlock()

    fields = StreamBlock([
        ('text', TextFieldBlock()),
        ('check', BooleanFieldBlock()),
        ('submit_button', SubmitButtonBlock()),
        ('image', ImageChooserBlock()),
        ('rich_text', RichTextBlock(features=['bold', 'italic'])),
    ])

    mandatory_field_blocks = [
        ('name', TextFieldBlock),
        ('message', TextFieldBlock),
    ]

    def form_valid(self, value, form):
        name = form.cleaned_data['name']
        message = form.cleaned_data['message']

        email_to = value['send_emails_to']

        send_mail(
            "message sent",
            f"message from {name}: {message}",
            'no-reply@domain.com',
            [email_to],
        )



from django.utils.functional import cached_property
from wagtail.blocks import ChooserBlock

from .forms import BaseFormBlock  # noqa
from .fields import *  # noqa


registered_forms = []


def reusable_form(name):
    def _register(form_block_class):
        registered_forms.append((name, form_block_class()))
        return form_block_class
    return _register


class ReusableFormChooserBlock(ChooserBlock):
    @cached_property
    def target_model(self):
        from formation.models import ReusableForm  # noqa
        return ReusableForm

    @cached_property
    def widget(self):
        from formation.widgets import ReusableFormChooser  # noqa
        return ReusableFormChooser()

    def get_form_state(self, value):
        return self.widget.get_value_data(value)

    class Meta:
        icon = 'form'
        template = 'formation/reusable_form.html'
